/*
 * Copyright 2008 Ayman Al-Sairafi ayman.alsairafi@gmail.com
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License
 *       at http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package jsyntaxpane.lexers;

import jsyntaxpane.*;
import java.util.List;
import javax.swing.text.Segment;

/**
 * A lexer that does nothing.  Used for plain document editing.
 * @author Ayman Al-Sairafi
 */
public class EmptyLexer implements Lexer {

	@Override
	public void parse(Segment segment, int ofst, List<Token> tokens) {
	}
}
