/*
 * Copyright 2008 Ayman Al-Sairafi ayman.alsairafi@gmail.com
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License
 *       at http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jsyntaxpane;

import java.io.IOException;
import jsyntaxpane.lexers.SimpleRegexLexer;

/**
 *
 * @author Ayman Al-Sairafi
 */
public class JavaRegexKit extends DefaultSyntaxKit {

    public JavaRegexKit() throws IOException {
        super(new SimpleRegexLexer("javaRegex.properties"));
    }
}